;*******************************************************************
; Le 22/09/2017 "Garage_a_pic_emetteur.asm"
;
; Pour gger 2 afficheurs et envoyer une trame rs232 vers le module
; de rception et de gestion des aiguillages.
;
; Trame envoye  1200 Bauds = '_' '_' 'D' 'U' 'D' 'U' 'G' 'O' (cr)(lf)  Exemple = __1212GO(+crlf)
; 10 caractres ASCII dont numro de garage : D=Dizaine, U=unit
; Le numro de voie de garage est en double pour tre contrle  l'arrive de la trame.
;
; On utilise un encodeur rotatif  deux sorties pour choisir le numro de garage.
; On valide le garage en appuyant sur le bouton de l'encodeur rotatif, ou sur un bouton plac  ct.
; Entres : RB6 : Phase (A) de l'encodeur rotatif
;         : RB5 : Phase (B) de l'encodeur rotatif
;         : RB4 : Bouton d'envoi la trame ASCII
;
; Les variables modifiables par l'utilisateur sont :
;   numgarageMIN	; Numro affich minimum du garage (0-98) 
;   numgarageMAX	; Numro affich maximum du garage (1-99)
;   numgarageMINX	; Numro du garage affich  la premire action sur action sur le bouton (+)
;   numgarageMAXX	; Numro du garage affich  la premire action sur action sur le bouron (-)
;
;
;*******************************************************************

        LIST P=16F628A, R=DEC    ; Use the PIC16F628 and decimal system
        #include "P16F628A.INC"  ; Include header file
        __config  _INTRC_OSC_NOCLKOUT & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_OFF & _CP_OFF & _MCLRE_OFF

;*******************************************************************
; Paramtres de configuration du programme modifiable suivant le nombre de garages disponibles.
;
numgarageMIN	EQU	 0	; Numro affich minimum du garage (0-98) 
numgarageMAX	EQU	19	; Numro affich maximum du garage (1-99)

numgarageMINX	EQU	 0	; Numro du garage affich  la premire action sur action sur le bouton (+)
numgarageMAXX	EQU	19	; Numro du garage affich  la premire action sur action sur le bouton (-)

;*******************************************************************
;  Port A
;   RA0 : Segment G
;   RA1 : Segment F
;   RA2 : Segment A
;   RA3 : Segment B
;   RA4 : Sortie collecteur ouvert pas utilise !
;   RA5 : Sortie collecteur ouvert pas utilise !
;   RA6 : Segment D
;   RA7 : Segment E
;
;  Port B
;   RB0 : Commun digit Unit
;   RB1 : RX sortie rs232, pas utilis mais configur en mme temps que RB2
;   RB2 : TX sortie rs232
;   RB3 : Segment C
;   RB4 : Entre GO= RB4
;   RB5 : Entre Encodeur rotatif phase (A)
;   RB6 : Entre Encodeur rotatif phase (B)
;   RB7 : Commun digit Dizaine
;
;*******************************************************************
        CBLOCK 0x20	; Declare variable addresses starting at 0x20
	numgarage	; Numro d'aiguillage d'affich en cours (0xFF  la mise sous tension)
	dizaine		; Afficheur des Dizaines  afficher
	unite		; Afficheur des Units  afficher
	bin		; Pour la routine de conversion binaire vers BDC
	hundreds	;       //
	tens_and_ones	;       //
	actionencours	; 0 = Pas d'action en cours
			; 1 = On affiche "Go" + on a active la sortie rs232
			; 2 = On affiche "Go" tant que le bouton GO n'est pas relach.
			; 3 = On affiche "Go" pendant quelques ms supplmentaires sur les afficheurs.
	carcpt		; Numro en cours du caractres envoys
	carunite	; Caractre ASCII envoy pour les units
	cardizaine	; Caractre ASCII envoy pour les dizaine
	flipflop	; Compteur pour afficher les afficheur Dizaine <-> Unit une fois sur deux  70 Hz
	pause1		; Compteur pour perdre du temps pour afficher GO pendant quelques ms
	pause2		;
	rotsave		; Sauvegarde la dernire position du codeur rotatif.
	rotactu		; Position actuel du codeur rotatif.
	rotaction	; Action suite lecture codeur rotatif
	rotcpt		; Compteur de phase (0  3) pour / 4 le nombre affich.
        ENDC

; BANK2 et BANK3 pas utiliss.

BANK0	macro				; passer en banque0
	bcf	STATUS,RP0
	endm
BANK1	macro				; passer en banque1
	bsf	STATUS,RP0
	endm

; Dfinitions de constante internes pour l'afficheur.
CARGOD	EQU	16	; = 'G'
CARGOU	EQU	17	; = 'o'
CARTIRET EQU	18	; = '-'

;*******************************************************************
;*******************************************************************
	ORG	0x000
	goto	init
	org	0x004
	goto	interrupt	; Les interruptions ne sont pas utilises.

init	movlw 7
	movwf	CMCON		; CMCON=7 comparateurs  l'arrt
	movlw	b'00000000'
	movwf	PORTA
	movlw	b'00000100'	; RB2(TX)=1
	movwf	PORTB

	BANK1			; RAM PAGE 1
        movlw	b'00000000'	; PortA en sortie
        movwf	TRISA
        movlw	b'01110110'	; PortB, RB7, RB3, RB0 en sortie
	movwf	TRISB
	movlw	51		; 1200 bauds = 51 et brgh = 0 = ; Boot Baud Rate = 1200, No Parity, 1 Stop Bit
	movwf	SPBRG
	movlw	b'00100000'	; 0 | 8 bits | tx enable | asynchrone | brgh = low = 0 | TRMT = TSR full | 9'bit = 0
	movwf	TXSTA		; enable Async Transmission, set brgh
	BANK0			; RAM PAGE 0
	movlw	b'10010000'	; port srie enable | RX9 | 
	movwf	RCSTA
	movlw   0xFF
	movwf   numgarage	; Valeur initiale 0XFF = Affichage = "--" = Pas de voie de garage de slectionne.
	movlw   0
	movwf   actionencours

	bcf 	STATUS,C	; Carry = 0
	movf	PORTB,w		
	andlw	b'01100000'
	movwf	rotsave
	swapf	rotsave,f	; rotsave = '000000XY'
	rrf	rotsave,f	; rotsave = Position de l'encodeur rotatif  la mise sous tension pour avoir un tat de dpart.
	clrf	rotcpt		; rotcpt = 0
	

;*******************************************************************
;*******************************************************************
debut	movf	numgarage,w
	sublw	0xFF
	btfss   STATUS,Z
	goto	pasff
	movlw   CARTIRET	; Valeur initiale de numgarage au dpart = 0xFF => Affichage = "--"
	movwf   unite
	movwf   dizaine
	call	binary_to_bcd	; Routine sans effet. Sert qu' quilibrer le temps perdu dans diffrentes parties du programme,
	goto	affdig		; pour avoir toujours la mme de frquence de raffraichissement des afficheurs.

pasff	movf	actionencours,w
	btfsc   STATUS,Z
	goto	pas_ff

aff_go	movlw   CARGOU		; Si actionencours != 0 => On affiche alors "Go"
	movwf   unite
	movlw   CARGOD
	movwf   dizaine
	call	binary_to_bcd	; Routine sans effet. Sert qu' quilibrer le temps perdu dans diffrentes parties du programme,
	goto	affdig

pas_ff	movf	numgarage,w	; actionencours = 0   =>  on affiche le numro de garage en cours
	movwf	bin
	call	binary_to_bcd	; Entre = bin -> Sorties = hundreds et tens_and_ones
	movf	tens_and_ones
	andlw	0x0F
	movwf	unite
	swapf	tens_and_ones,w
	andlw	0x0F
	movwf	dizaine
;
; Gestion des afficheurs, pour afficher les chiffres sur l'afficheur.
;

affdig	incf	flipflop
	btfss	flipflop,6	; flipflop(bit n6) = Frquence de rafraichissement = 60Hz.
	goto	affdiz
	movf	unite,w	
	sublw	18		; Si Unite > Indice maximum de la table "digit_table", on affiche '-'.
	btfsc   STATUS,C
	goto	suite1
	movlw	b'00000001' 	; Afficher '-' = Tiret.
	goto	suite2

suite1	movlw	0x07
	movwf	PCLATH
	movf	unite,w	
	call	digit_table
suite2	bcf	PORTB,7		; RB0=7 Pour teindre le transistor des dizaines
	movwf	PORTA		; Pour afficher tous les segments sauf le 'C'.
	andlw	b'00100000'
	btfss   STATUS,Z
	bsf	PORTB,3		; Pour afficher le segment 'C'.
	btfsc   STATUS,Z
	bcf	PORTB,3		; Pour teindrele segment 'C'.
	bsf	PORTB,0		; RB7=1 Pour allumer le transistor des units = on.
	goto	in0

affdiz	movf	dizaine,w	
	sublw	18		; Si Dizaine > Indice maximum de la table "digit_table", on affiche '-'.
	btfsc   STATUS,C
	goto	suite4
	movlw	b'00000001' 	; Afficher '-' = Tiret.
	goto	suite5

suite4	movlw	0x07
	movwf	PCLATH
	movf	dizaine,w	
	btfsc   STATUS,Z
	goto	suite5		; Grosse astuce si dizaine = 0, on teint le digit des dizaines.
	call	digit_table
suite5	bcf	PORTB,0		; RB7=0 Pour le transistor des units, on teint les 2 afficheurs.
	movwf	PORTA		; Pour afficher tous les segments sauf le 'C'.
	andlw	b'00100000'
	btfss   STATUS,Z
	bsf	PORTB,3		; Pour afficher le segment 'C'.
	btfsc   STATUS,Z
	bcf	PORTB,3		; Pour teindre le segment 'C'.
	bsf	PORTB,7		; RB0=1 Pour allumerle transistor des dizaines = on.
;
; Gestion des entres
;   B6 + B5 = Encodeur rotatif   
;   B4      = Bouton GO
;
; Variable "actionencours" 
;  0 = pas d'action en cours, si l'on active le bouton GO -> 1.
;  1 = on a traite la commande et l'on envoie la trame ascii,
;      Aprs le dernier caractre envoy -> 2
;  2 = on affiche "Go on attend que l'entre du bouton GO soit  zro pour -> 0
;  3 = on affiche "Go pendant encore quelques ms pour -> 0
;
in0	movf	actionencours,w
	btfss   STATUS,Z	
	goto	in1		; actionencours != 0 

	bcf 	STATUS,C	; actionencours = 0 => pas d'action en cours, donc on peut lire l'encodeur rotatif  :  Carry = 0
	movf	PORTB,w
	andlw	b'01100000'
	movwf	rotactu
	swapf	rotactu,f	; rotsactu = '000000XY'
	rrf	rotactu,f	; rotsactu = Position de l'encodeur rotatif.

	rlf	rotsave,f
	rlf	rotsave,w
	iorwf	rotactu,w
	movwf	rotaction	; rotaction = de 0000  1111.

	movf	rotactu,w
	movwf	rotsave

	movlw	0x07
	movwf	PCLATH
	movf	rotaction,w	; w = '0000XXYY" = rotsave(2 bits) + rotactu(2 bits)
	call	rot_table
	movwf	rotaction	; Retour, w = 0, 1 ou 0xFF
;
; Le commuteur rotatif, commute les 4 tats entre deux positions stable du bouton.
; On utilise la variable "rotcpt" pour diviser par 4 les vnements de changement de phase.
; Avec un commutateur rotatif simple tat, il faudrait supprimer les lignes de code utilisant "rotcpt".
;
	movlw	0x03
	andwf	rotcpt,f	; On garde 2 bits du compteur pour ne compter qu'une fois sur 4.

	movf	rotaction,w
	sublw	1
	btfss   STATUS,Z	
	goto	rotp1		; rotaction = 1
	incf	rotcpt,f
	btfss	rotcpt,2	
	goto	in0_4
	goto	in_0_0		; dpassement rotcpt sur 2 bits = on a avanc d'un cran

rotp1	movf	rotaction,w
	sublw	0xFF
	btfss   STATUS,Z	
	goto 	in_0_0		; rotaction = 0xFF
	decf	rotcpt,f
	btfss	rotcpt,2	
	goto	in0_4
	goto	in_0_0		; dpassement rotcpt sur 2 bits = on a recul d'un cran

;----------------------------------	
; ENTREE = (+)
;----------------------------------
in_0_0	movf	rotaction,w
	sublw	1
	btfss   STATUS,Z	
	goto	in0_2
	movf	numgarage,w
	sublw	0xFF
	btfss   STATUS,Z
	goto	in0_1
	movlw	numgarageMIN	; Si 1er passage '+1' : numgarageMIN -> numgarage
	movwf	numgarage
	goto	infin		; Action en cours reste  0
in0_1	movf	numgarage,w
	sublw	numgarageMAX-1
	btfss   STATUS,C
	goto	infin		; numgarage = numgarageMIN, Action en cours reste  0
	incf	numgarage
	goto	infin		; Action en cours reste  0
;----------------------------------
; ENTREE = (-)
;----------------------------------
in0_2	
	movf	rotaction,w
	sublw	0xFF
	btfss   STATUS,Z	
	goto	in0_4
	movf	numgarage,w
	sublw	0XFF
	btfss   STATUS,Z
	goto	in0_3
	movlw	numgarageMAX	; Si 1er passage '+1' : numgarageMAX-> numgarage
	movwf	numgarage
	goto	infin		; Action en cours reste  0
in0_3	movlw	numgarageMIN+1
	subwf	numgarage,w
	btfss   STATUS,C
	goto	infin		; numgarage = numgarageMIN, Action en cours reste  0
	decf	numgarage
	goto	infin		; Action en cours reste  0
;----------------------------------
; ENTREE = (B4) = (GO)
;----------------------------------
in0_4	btfss   PORTB,4	
	goto	infin		
	movf	numgarage,w	; PortB = 1
	sublw	0XFF
	btfsc   STATUS,Z
	goto	infin		; Pas de numro de garage valide
	clrf	carcpt		; n du caractres  envoyer
	movf	numgarage,w	; valeur numrique
	movwf	bin
	call	binary_to_bcd	; bin -> hundreds + tens_and_ones
	movf	tens_and_ones
	andlw	0x0F
	addlw	'0'
	movwf	carunite
	swapf	tens_and_ones,w
	andlw	0x0F
	addlw	'0'
	movwf	cardizaine
	goto	inx_a1		; Action en cours 0 -> 1

; ------------------------------------------------------------------
; ------------------------------------------------------------------
; Actionencours = 1
; On envoi 10 caractres ASCI sur la sortie RS232
;
; Trame envoye  1200 Bauds = '_' '_' 'D' 'U' 'D' 'U' 'G' 'O' (cr)(lf)  Exemple = __1212GO(+crlf)
; 10 caractres ASCII dont numro de garage : D=Dizaine, U=unit
; Le numro de voie de garage est en double pour tre contrle  l'arrive de la trame.
;
in1	movf	actionencours,w
	sublw	1
	btfss   STATUS,Z	
	goto	in2	

	BANK1			; RAM PAGE 1    Actionencours = 1
	btfsc	TXSTA,TRMT	;
	goto	in1_00
	BANK0			; RAM PAGE 0
	goto	infin		; Le dernier caractre n'est pas encore parti.

in1_00	BANK0			; RAM PAGE 0
	movf	carcpt,w	; carcpt = compteur de caractre envoy de 0  9
	sublw	0
	btfss   STATUS,Z	
	goto	in1_01
	movlw	'_'		; '_'
	goto	in1_20

in1_01	movf	carcpt,w
	sublw	1
	btfss   STATUS,Z	
	goto	in1_02
	movlw	'_'		; '_'
	goto	in1_20

in1_02	movf	carcpt,w
	sublw	2
	btfss   STATUS,Z
	goto	in1_03
	movf	cardizaine,w	; '0' Dizaine
	goto	in1_20

in1_03	movf	carcpt,w
	sublw	3
	btfss   STATUS,Z	
	goto	in1_04
	movf	carunite,w	; '0' Unit
	goto	in1_20

in1_04	movf	carcpt,w
	sublw	4
	btfss   STATUS,Z	
	goto	in1_05
	movf	cardizaine,w	; '0' Dizaine
	goto	in1_20

in1_05	movf	carcpt,w
	sublw	5
	btfss   STATUS,Z	
	goto	in1_06
	movf	carunite,w	; '0' Unit
	goto	in1_20

in1_06	movf	carcpt,w
	sublw	6
	btfss   STATUS,Z	
	goto	in1_07
	movlw	'G'		; 'G'
	goto	in1_20

in1_07	movf	carcpt,w
	sublw	7
	btfss   STATUS,Z	
	goto	in1_08
	movlw	'O'		; 'O'
	goto	in1_20

in1_08	movf	carcpt,w
	sublw	8
	btfss   STATUS,Z	
	goto	in1_09
	movlw	0x0D ; CR	; (cr)  retour chariot
	goto	in1_20

in1_09	movf	carcpt,w
	sublw	9
	btfss   STATUS,Z	
	goto	in1_10
	movlw	0x0A ; LF	; (lf)  saut de ligne
	goto	in1_20

in1_10	goto	inx_a2		; On a tout envoy : actionencours 1 -> 2 

in1_20	movwf	TXREG		; Envoi de la donne dans w.
	incf	carcpt
	goto	infin
; ------------------------------------------------------------------
; ------------------------------------------------------------------

in2	movf	actionencours,w
	sublw	2
	btfss   STATUS,Z	
	goto	in3
	movf	PORTB,w		; Actionencours = 2
	andlw	b'00010000'	; Attente que la tension soit  0 sur l'entre GO.
	btfss   STATUS,Z	
	goto	infin		; L'entre GO est encore  1, le bouton est encore enfonc.
	movlw	255
	movwf	pause1
	movlw	40		;	40 = 1 sec  , 100 = 2,7 sec
	movwf	pause2
	goto	inx_a3		; L'entre GO est passe  0, action en cours 2 -> 3

in3	movf	actionencours,w
	sublw	3
	btfss   STATUS,Z	
	goto	infin
			
	decfsz	pause1		; Actionencours = 3
	goto	infin		; On attend quelques ms avant de repasser action en cours 3 -> 0
	decfsz	pause2		; Actionencours = 3
	goto	infin		; On attend quelques ms avant de repasser action en cours 3 -> 0
	goto	inx_a0


inx_a0	clrf	actionencours	; Action en cours X -> 0
	goto	infin
inx_a1	movlw	1
	movwf	actionencours	; Action en cours X -> 1
	goto	infin
inx_a2	movlw	2
	movwf	actionencours	; Action en cours X -> 2
	goto	infin
inx_a3	movlw	3
	movwf	actionencours	; Action en cours X -> 3
	goto	infin
	nop

infin:	goto	debut

;*******************************************************************
interrupt return 		; Interruptions pas utilises.

;*******************************************************************
; Routine de conversion binaire vers le format BCD : binary_to_bcd - 8-bits
; http://www.piclist.com/techref/microchip/math/radix/b2bhp-8b3d.htm
; Binary to BCD half-packed 8 bit to 3 digit
; From: Scott Dattalo, notes
; Comments added by Alex Forencich
;
; Input
;    bin  - 8-bit binary number
;    A1*16+A0
; Outputs
;    hundreds - the hundreds digit of the BCD conversion
;    tens_and_ones - the tens and ones digits of the BCD conversion
;
binary_to_bcd:
	clrf	hundreds
	swapf	bin,W		; swap the nibbles
	addwf	bin,W		; so we can add the upper to the lower
	andlw	B'00001111'	; lose the upper nibble (W is in BCD from now on)
	btfsc	STATUS,DC	; if we carried a one (upper + lower > 16)
	addlw	0x16		; add 16 (the place value) (1s + 16 * 10s)
	btfsc	STATUS,DC	; did that cause a carry from the 1's place?
	addlw	0x06		; if so, add the missing 6 (carry is only worth 10)
	addlw	0x06		; fix max digit value by adding 6
	btfss	STATUS,DC	;	if was greater than 9, DC will be set
	addlw	-0x06		; if if it wasn't, get rid of that extra 6
	btfsc	bin,4		; 16's place
	addlw	0x16 - 1 + 0x6	; add 16 - 1 and check for digit carry
	btfss	STATUS,DC
	addlw	-0x06		; if nothing carried, get rid of that 6
	btfsc	bin,5		; 32nd's place
	addlw	0x30		; add 32 - 2
	btfsc	bin,6		; 64th's place
	addlw	0x60		; add 64 - 4
	btfsc	bin,7		; 128th's place
	addlw	0x20		; add 128 - 8 % 100
	addlw	0x60		; has the 10's place overflowed?
	rlf	hundreds,F	; pop carry in hundreds' LSB
	btfss	hundreds,0	; if it hasn't
	addlw	-0x60		; get rid of that extra 60
	movwf	tens_and_ones	; save result
	btfsc	bin,7		; remeber adding 28 - 8 for 128?
	incf	hundreds,F	; add the missing 100 if bit 7 is set
	return			;	all done!


;*******************************************************************
; Pour alimenter les segments de l'afficheur en fontion du numro affich
;
;	Digit	E	D	C	x	B	A	F	G
;	Bit	A7	A6	B3!	A4	A3	A2	A1	A0
;    Valeur								
;	0	1	1	1	x	1	1	1	0
;	1	0	0	1	x	1	0	0	0
;	2	1	1	0	x	1	1	0	1
;	3	0	1	1	x	1	1	0	1
;	4	0	0	1	x	1	0	1	1
;	5	0	1	1	x	0	1	1	1
;	6	1	1	1	x	0	1	1	1
;	7	0	0	1	x	1	1	0	0
;	8	1	1	1	x	1	1	1	1
;	9	0	1	1	x	1	1	1	1
;	10 'A'	1	0	1	x	1	1	1	1
;	11 'B'	1	1	1	x	0	0	1	1
;	12 'C'	1	1	0	x	0	1	1	0
;	13 'D'	1	1	1	x	1	0	0	1
;	14 'E'	1	1	0	x	0	1	1	1
;	15 'F'	1	0	0	x	0	1	1	1
;	16 'G'	1	1	1	x	0	1	1	1	= "G"
;	17 'o'	1	1	1	x	0	0	0	1	= "o"
;	18 '-'	0	0	0	x	0	0	0	1	= "-"
;	19 ' '	0	0	0	x	0	0	0	0	= " "
;	20 '?'	0	1	0	x	0	1	0	1	= "?"
;
;
;*******************************************************************
; Sous programme "digit_table", conversion binaire vers afficheur.
; Entre w = 0  20 = Caratre  afficher
; Retour w = Valeurs des 7 segments  afficher
;
; Utilisation de la table :
; 	movlw	0x07
;	movwf	PCLATH
;	movf	unite,w	
;	call	digit_table

	ORG 0x700		; Adresse des tableaux en fin de mmoire (moins de 256 octets pour les deux tables)
digit_table: 
	addwf	PCL,f		; Ajouter w  PCL
	retlw	b'11101110'	;  0 = '0'
	retlw	b'00101000'	;  1 = '1'
	retlw	b'11001101'	;  2 = '2'
	retlw	b'01101101'	;  3 = '3'
	retlw	b'00101011'	;  4 = '4'  
	retlw	b'01100111'	;  5 = '5'
	retlw	b'11100111'	;  6 = '6' 
	retlw	b'00101100'	;  7 = '7' 
	retlw	b'11101111'	;  8 = '8' 
	retlw	b'01101111'	;  9 = '9' 
	retlw	b'10101111'	; 10 = 'A' 
	retlw	b'11100011'	; 11 = 'B'
	retlw	b'11000110'	; 12 = 'C'
	retlw	b'11101001'	; 13 = 'D'
	retlw	b'11000111'	; 14 = 'E'
	retlw	b'10000111'	; 15 = 'F'
	retlw	b'11100110'	; 16 = 'G'
	retlw	b'11100001'	; 17 = 'o'
	retlw	b'00000001'	; 18 = '-' = Tiret
	retlw	b'00000000'	; 19 = ' ' = Espace
	retlw	b'01000101'	; 20 = '=' = Fin de table, caractre non prvu

rot_table:			; Pour retourner la valeur  additionner  (moins de 256 octets pour les deux tables)
	addwf	PCL,f		; Ajouter w  PCL
	RETLW   H'00'   ; 00 -> 00
	RETLW   H'FF'   ; 00 -> 01 -1
	RETLW   H'01'   ; 00 -> 10 +1
	RETLW   H'00'   ; 00 -> 11
	RETLW   H'01'   ; 01 -> 00 +1
	RETLW   H'00'   ; 01 -> 01
	RETLW   H'00'   ; 01 -> 10 
	RETLW   H'FF'   ; 01 -> 11 -1
	RETLW   H'FF'   ; 10 -> 00 -1
	RETLW   H'00'   ; 10 -> 01
	RETLW   H'00'   ; 10 -> 10
	RETLW   H'01'   ; 10 -> 11 +1
	RETLW   H'00'   ; 11 -> 00
	RETLW   H'01'   ; 11 -> 01 +1
	RETLW   H'FF'   ; 11 -> 10 -1
	RETLW   H'00'   ; 11 -> 11
	END