;*******************************************************************
; Le 08/10/2017 Garage_a_pic_recepteur.asm
;
; Pour le module rcepteur : Garage_a_pic_recepteur.asm
;
; Programme de commande des aiguillages  la rception d'une trame RS232.
; Recoit une trame rs232 depuis le module metteur.
; Trame reuee  1200 Bauds = '_' '_' 'D' 'U' 'D' 'U' 'G' 'O' (cr)(lf)  Exemple = __1212GO(+crlf)
; = 10 caractres dont : D=Dizaine, U=Unit
; Le numro de voie de garage est en double pour tre contrle  l'arrive de la trame.
;
; Les paramtres modifiables sont 
;  - Les valeurs des temporisations dans table de dlais : tmpd10
;  - La dclaration des aiguillages  activer par garage dans la table : tabaig
;  - Les variables "garageMIN" et "garageMAX" sont les numros valides pour le premier et dernier garage.
; 
; Maximum de 32 aiguillages et 99 garages (80 max si mmorisation des manoeuvres)
; N d'aiguillage =  0  32 sur 5 bits = 16*bit_convertit_en_GO2 + 8*RELAIS1 + 4*not(RELAIS0) + 2*not(RELAIS4) + 1*not(RELAIS5)
; N d'aiguillage =  0  16 si l'on utilise Carte AIG RELAIS_GO1 (IN6)
; N d'aiguillage = 17  32 si l'on utilise Carte AIG RELAIS_GO2 (IN3)
; Position  droit ou dvi = RELAIS2.
;
;*******************************************************************

        LIST P=16F628A, R=DEC    ; Use the PIC16F628 and decimal system
        #include "P16F628A.INC"  ; Include header file
        __config  _INTRC_OSC_NOCLKOUT & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_OFF & _CP_OFF & _MCLRE_OFF

;*******************************************************************
; Paramtres de configuration du programme modifiable suivant le nombre de garages disponibles.
;
garageMIN	EQU	 0	; Numro du premier garage
garageMAX	EQU	19	; Numro du dernier garage
;
;*******************************************************************
;  Port A
;   RA0 : Carte AIG RELAIS_2
;   RA1 : Carte AIG RELAIS_GO2 (IN3)
;   RA2 : Out1 MUX
;   RA3 : Out2 MUX
;   RA4 : In1 MUX (lecture cavaliers)
;   RA5 : In2 MUX    //
;   RA6 : Carte AIG RELAIS_1
;   RA7 : Carte AIG RELAIS_GO1 (IN6)
;
;  Port B
;   RB0 : In : Mmorisation des action sur les aiguillages = Marche/arrt
;   RB1 : RX sortie rs232
;   RB2 : TX sortie rs232, pas utilis mais configur en mme temps que RB1
;   RB3 : Led de tmoin d'activit (clignote)
;   RB4 : Led retour info vers rcepteur
;   RB5 : carte AIG RELAIS_4
;   RB6 : carte AIG RELAIS_0
;   RB7 : carte AIG RELAIS_5
;
bitGO12		EQU	4	;   // = bit n4
POS0		EQU	0x00	; Aiguillage en position 0 = bit n 6
POS1		EQU	0x40	; Aiguillage en position 1 = bit n6
bitPOS1		EQU	6	;   // = bit n6
POSFIN		EQU	0x80	; Fin de description d'un garage = bits n7
bitPOSFIN	EQU	7	;   // = bit n7
POSFINFIN	EQU	0xFF	; Fin de table
;
#DEFINE 	RELAIS0		PORTB,6
#DEFINE 	RELAIS1		PORTA,6
#DEFINE 	RELAIS2		PORTA,0
#DEFINE 	RELAIS4		PORTB,5
#DEFINE 	RELAIS5		PORTB,7
#DEFINE 	RELAISGO1	PORTA,7
#DEFINE 	RELAISGO2	PORTA,1

        CBLOCK 0x20	; Adresse de dpart des donnes en BANK0
	garage		; numro du garage reu en rs232
	garageind	; numro de garage en cours de lecture dans la table

	aigbrut		; Aiguillage lu dans la table
	tabind		; Indice lecture de table
	carind		; n caractre reu. ( 0  9)

	car
	car01		; Premier caractre rs232 reu, autre que '_'
	car02		;     //
	car03		;     //
	car04		; Dernier caractre reu.
	tmp1
	tcharge		; Dure de recharger entre deux impulsions sur 2 bits. 0=100 ms, 1=200ms, 2=300ms, 3=400ms.
	tdecharge	; Dure de dcharge de l'impulsion sur 2 bits.         0=100 ms, 1=200ms, 2=300ms, 3=400ms.
	cpt1
	cpt2
	cpt3
	flipflop1	; Pour faire clignoter une led de tmoin d'activit du PIC
	flipflop2	;     //
	memoire		; Mmorisation des actions sur les aiguillages = Marche/arrt = (0-1).
	numtable	; Numro de table en cours =(0-1).
        ENDC

        CBLOCK	0xA0	; Adresse de dpart des donnes en BANK1
	aigpos : 80	; Tableau de la position en mmoire des aiguillages 1 bit = 1 aiguillage
			; 80 aiguillages maxi. 2 bits par aiguillage : bit(1) = 1 => dja utilis | bit(0) => position de l'aiguillage
        ENDC

; BANK2 et BANK3 pas utiliss.

BANK0	macro			; passer en banque0
	bcf	STATUS,RP0
	endm
BANK1	macro			; passer en banque1
	bsf	STATUS,RP0
	endm

	ORG	0x000		; Program starts at 0x000
	goto	init
	org	0x004
	goto	interrupt

init	movlw 7
        movwf	CMCON		; CMCON=7 set comperators off
        movlw	b'00000000'	; set up portA
        movwf	PORTA
        movlw	b'00000100'	; RB2(TX)=1 others are 0
        movwf	PORTB

        BANK1			; RAM PAGE 1
        movlw	b'00110000'	; PortA en sortie, RA4 et RA5 en entre
        movwf	TRISA
        movlw	b'00000111'	; PortB en sortie, RB0, RB1 et RB2 en entre
        movwf	TRISB

; ------------------------------------
; Boot Baud Rate = 1200, No Parity, 1 Stop Bit
;
	movlw	51		; 1200 bauds = 51 et brgh = 0
	movwf	SPBRG
	movlw	b'00100000'	; 0 | 8 bits | tx enable | asynchrone | brgh = low = 0 | TRMT = TSR full | 9'bit = 0
        movwf	TXSTA		; enable Async Transmission, set brgh
	BANK0			; RAM PAGE 0
	movlw	b'10010000'	; port srie enable | RX9 | 
	movwf	RCSTA
; ------------------------------------

	clrf	tmp1
wait1	decfsz	tmp1,F		; Petite tempo d'entre pour stabiliser le buffer d'entre rs232
	goto	wait1
	movf    RCREG,W
	movf    RCREG,W
	movf    RCREG,W		; Efface le tampon d'entre RS232
	clrf	carind		; Raz indice du caractre reu.

	BANK1			; RAM PAGE 1
	movlw	aigpos		; Efface toute la mmoire en banque1 => aigpos[x]=0
	movwf	FSR
raz1	clrf	INDF
	incf	FSR,f
	movf	FSR,w
	sublw	0xF0
	btfss	STATUS,Z	
	goto	raz1
	BANK0			; RAM PAGE 0
;
;*******************************************************************
;*******************************************************************

debut	decfsz	flipflop1	
	goto	debut2
	bsf	PORTB,3		; flipflop1 = 0
	decf	flipflop2
	btfsc	flipflop2,7
	bcf	PORTB,3		; flipflop2(bit 7) => RB3 => Fait clignoter  1Hz une led tmoin d'activit sur le port RB3

;*******************************************************************
	bcf	PORTA,2		; Lecture des cavaliers pour choisir le temps de charge et de dcharge du condensateur
	bsf	PORTA,3		; Sorties = RA2=0 et RA3=1. Lecture des entres = RA4 et RA5.
	clrf	tcharge		; Tempo = 4 instructions avant lecture des entres.
	clrf	tdecharge
	nop
	nop
	nop
	nop
	btfsc	PORTA,4
	incf	tcharge,f
	btfsc	PORTA,5
	incf	tdecharge,f

	bcf	PORTA,3		; Sorties = RA2=1 et RA3=0. Lecture des entres = RA4 et RA5.
	bsf	PORTA,2
	nop			; Tempo = 4 instructions avant lecture des entres.
	nop
	nop
	nop
	nop	
	movlw	2
	btfsc	PORTA,4
	addwf	tcharge		; Dure de charge du condensateur sur 2 bits.   0=100 ms, 1=200ms, 2=300ms, 3=400ms.
cava1	btfsc	PORTA,5
	addwf	tdecharge	; Dure de dcharge du condensateur sur 2 bits. 0=100 ms, 1=200ms, 2=300ms, 3=400ms.
	bcf	PORTA,2
	
	btfsc	PORTB,0		; Lecture du cavaliers pour choisir d'activer la mmorisation des actions sur les aiguillages = Marche/arrt
	goto	cava3		; RBA(0)=1 => Il ne faut pas mmoriser les actions !

	btfsc	memoire,0	; RBA(0)=0 => Il faut mmoriser les actions !  Flag mmoire dj activ ?  
	goto	cava4		; Oui ! => Rien  faire. memoire dj=1 !
	bsf	memoire,0	; Non ! => Raz table + Set Flag memoire=1 !

	BANK1			; RAM PAGE 1
	movlw	aigpos		; Efface toute la mmoire en banque1 => aigpos[x]=0
	movwf	FSR
raz2	clrf	INDF
	incf	FSR,f
	movf	FSR,w
	sublw	0xF0
	btfss	STATUS,Z	
	goto	raz2
	BANK0			; RAM PAGE 0
	goto	cava4

cava3	bcf	memoire,0	; RBA(0)=1 => Il ne faut pas mmoriser les actions ! => memoire=0 !
cava4	nop

;*******************************************************************
; Reu un caractre ?
debut2	btfss	RCSTA,OERR	; Vrifie si le buffer d'entre rs232 est satur ?
	goto	car_buf		; Non !

; Traitement de la saturation du buffer d'entre rs232
	bcf	RCSTA,CREN	; Dans ce cas, on efface le bit OERR (0->CREN) et
	bsf	RCSTA,CREN	; on relance la rception.
	movf    RCREG,w
	movf    RCREG,w		; Efface le tampon d'entre RS232.
	goto	carclr		; On efface tout.

car_buf	btfss	PIR1,RCIF	; A t'on reu un octet ?
	goto	carfin		; Non !

car_rec	movf	RCREG,w		; Sauvegarde de l'octet reu -> car.
	movwf	car
	sublw	'_'
	btfsc	STATUS,Z
	goto	carclr		; Si car='_' => Reset de l'indice du nombre de caractre reu.

;*******************************************************************

car_10	incf	carind,f	; Indice du caractre reu++
	movf	carind,w
	sublw	4
	btfss	STATUS,C	; Indice du caractre reu >= 4 ?
	goto	car_15		; Oui !

	movf	car,w
	addlw	-'0'
	btfss	STATUS,C
	goto	carclr		; car < '0' = le 1,2,3,4 caractre n'est pas un chiffre. Erreur => Reset
	movf	car,w
	sublw	'9'
	btfss	STATUS,C
	goto	carclr		; car > '9' = le 1,2,3,4 caractre n'est pas un chiffre. Erreur => Reset

	movf	carind,w	; Le 1,2,3,4 caractre reu est bien un chiffre en ascii.
	sublw	1
	btfss	STATUS,Z
	goto	car_12
	movf	car,w		; car -> car01 (1er caractre reu)
	movwf	car01
	goto	carfin
;
car_12	movf	carind,w
	sublw	2
	btfss	STATUS,Z
	goto	car_13
	movf	car,w		; car -> car02(2me caractre reu)
	movwf	car02
	goto	carfin
;
car_13	movf	carind,w
	sublw	3
	btfss	STATUS,Z
	goto	car_14
	movf	car,w		; car -> car03 (3me caractre reu)
	movwf	car03
	goto	carfin
;
car_14	movf	carind,w
	sublw	4
	btfss	STATUS,Z
	goto	carfin
	movf	car,w		; car -> car04 (4me caractre reu)
	movwf	car04
	goto	carfin

car_15 	movf	carind,w
	sublw	5
	btfss	STATUS,Z
	goto	car_16
	movf	car,w		; L'indice du caractre reu= !
	sublw	'G'
	btfss	STATUS,Z
	goto	carclr		; Le 5 caractre n'est pas 'G'. Erreur => Reset
	goto	carfin

car_16 	movf	carind,w
	sublw	6
	btfss	STATUS,Z
	goto	car_17
	movf	car,w		; L'indice du caractre reu=6.
	sublw	'O'
	btfss	STATUS,Z
	goto	carclr		; Le 6 caractre n'est pas 'O'. Erreur => Reset
	goto	carfin

car_17 	movf	carind,w
	sublw	7
	btfss	STATUS,Z
	goto	car_18
	movf	car,w		; L'indice du caractre reu=7 !
	sublw	0x0D
	btfss	STATUS,Z
	goto	carclr		; Le 7 caractre n'est pas 'cr'. Erreur => Reset
	goto	carfin

car_18 	movf	carind,w
	sublw	8
	btfss	STATUS,Z
	goto	carclr		; On n'attend pas plus de 8 caractres. Erreur => Reset
	movf	car,w		; L'indice du caractre reu=8 !
	sublw	0X0A
	btfss	STATUS,Z
	goto	carclr		; Le 8 et dernier caractre n'est pas 'lf' Erreur => Reset
	goto	cartrait	; Dernier caractre reu. Traitement de la trame reue.

carclr	clrf	carind		; On fait un raz de tout ce qui tait reu.
	movlw	0xFF
	movwf	car01
	movwf	car02
	movwf	car03
	movwf	car04
carfin	goto	debut

;*******************************************************************
; Traitement de la trame reue et dcodage du numro de garage en ASCII vers binaire.
; La trame reue se composait bien de '_' + '_' + 4 ascii + 'G' + 'O' + cr + lf
; Entre = car01 + car02 + car03 + car04 
; Sortie = numro du garage en binaire.

cartrait movf	car01,w
	subwf	car03,w
	btfss	STATUS,Z
	goto	carclr		; Si car01 # car03, on raz tout (n de dizaine diffrent).
	
	movf	car02,w
	subwf	car04,w
	btfss	STATUS,Z
	goto	carclr		; Si car02 # car04, on raz tout (n d'unit diffrent). 

; 2 digit ASCII decimal to 8 bit Binary 
; http://www.piclist.com/techref/microchip/math/radix/a2b-2d8b-sd.htm
	rlf	car01,w
	andlw	0x1e		; w=2*tens
	movwf	tmp1
	rlf	tmp1,f		; temp=4*tens
	rlf	tmp1,f		; temp=8*tens
	addwf	car02,w
	addlw	-'0'		; convert from ASCII to decimal
				; w=tens*2 + ones
	addwf	tmp1,w		; w=10*tens + ones
	movwf	garage		; w -> garage

	movlw	garageMIN
	subwf	garage,w
	btfss   STATUS,C
	goto	carclr		; si garage < garageMIN. Erreur => Reset
	movf	garage,w
	sublw	garageMAX
	btfss	STATUS,C
	goto	carclr		; si garage > garageMAX. Erreur => Reset

;********************************************************************
; Positionnement des aiguillages en fonction de la variable "garage".

	clrf	tabind		; Indice de lecture des lments du tableau des garages (0-255).
	clrf	garageind	; Indice du garage en cours de lecture dans la table (0-99).
	clrf	numtable	; Numro de table en cours de lecture (0-1).

bcltbl1	movlw	0x05
	movwf	PCLATH
	movf	tabind,w
	call	tabaig1		; tabaig[garageind] -> w      Table des garages, correspondance avec les aiguillages.
	movwf	aigbrut 	; aigbrut = Numro aiguillage(5 bits) + POS0/1(0x00/0x40)

	sublw	POSFINFIN
	btfsc	STATUS,Z	; aigbrut=POSFINFIN(0xFF) ?
	goto	bcltb02		; On est en fin de table1, on passe  la table suivante.

	movf	garage,w	
	subwf	garageind,w
	btfsc	STATUS,Z	; ngarage en cours = ngarage recherch ?
	goto	bclaig		; GOTO : On a trouv le 1er lment du tableau pour le garage recherch !!!

	movf	aigbrut,w
	andlw	POSFIN
	btfss	STATUS,Z	; aigbrut=POSFIN(0x80) ?
	incf	garageind,f	; garageind++  =>  On passe au garage suivant.
	incf	tabind,f	; Dans tous les cas : tabaig1[tabind++] = > On passe  l'lment suivant dans la table.
	goto	bcltbl1

bcltb02	clrf	tabind		; Indice de lecture des lments du tableau des garages.
	incf	numtable,f	; 2eme table en cours

bcltbl2	movlw	0x06
	movwf	PCLATH
	movf	tabind,w
	call	tabaig2		; tabaig[garageind] -> w      Table des garages, correspondance avec les aiguillages.
	movwf	aigbrut 	; aigbrut = Numro aiguillage(5bits) + POS0/1(0x00/0x40)

	sublw	POSFINFIN
	btfsc	STATUS,Z	; aigbrut=POSFINFIN(0xFF) ?
	goto	carclr		; On est en fin de 2eme table et l'on n'a pas trouv ce garage. Erreur => Reset

	movf	garage,w	
	subwf	garageind,w
	btfsc	STATUS,Z	; ngarage en cours = ngarage recherch ?
	goto	bclaig		; GOTO : On a trouv le 1er lment du tableau pour le garage recherch !!!

	movf	aigbrut,w
	andlw	POSFIN
	btfss	STATUS,Z	; aigbrut=POSFIN(0x80) ?
	incf	garageind,f	; garageind++  =>  On passe au garage suivant.
	incf	tabind,f	; Dans tous les cas : tabaig2[tabind++] = > On passe  l'lment suivant dans la table.
	goto	bcltbl2


;*****************************************************************************************
; tabaig1/2[tabind] => OK. On a trouv le 1er lment en mmoire pour le garage recherch.
;
bclaig	bsf	PORTB,3		; On allume en permanence la led tmoin d'activit branche sur le port RB3.
	btfsc	aigbrut,bitPOSFIN ; aigbrut=POSFIN(0x80) ?
	goto	tblfin		; On est en fin de table pour ce garage. Plus d'action  faire => tblfin
	movf	aigbrut,w	
	sublw	POSFINFIN
	btfsc	STATUS,Z	; aigbrut=POSFINFIN(0xFF) ?
	goto	tblfin		; On est en fin de table. Plus d'action  faire => tblfin

; Si numro d'aiguillage 00  15, action => GO1
; Si numro d'aiguillage 16  31, action => GO2
; Actionne les relais suivant le n d'aiguillage.
;
; On vrifie si l'on a pas dj mis l'aiguillage dans la bonne position au moins une fois ?
;
	bsf	PORTB,4		; On allume les points des afficheurs de l'metteur branchs sur le port 4.
	btfss	memoire,0	; Mmorisation des actions sur les aiguillages, memoire=1 ?
	goto	mem2		; Non !

	movf	aigbrut,w	; 
	andlw	b'00111111'	; w = N d'aiguillage en cours -> FSR
	addlw	aigpos		; (w) = aigpos + n aiguillage en cours -> FSR
	movwf	FSR

	btfss	INDF,1		; aipos[x]Aiguillage dja utilis ?
	goto	mem2		; Non !

	btfsc	aigbrut,bitPOS1	; Aiguillage dans la bonne position ?
	goto	mem1

	btfss	INDF,0		; Aiguillage dans la bonne position 0 ?
	goto	aigsuit
	goto	mem2

mem1	btfsc	INDF,0		; Aiguillage dans la bonne position 1 ?
	goto	aigsuit

;                                                     _______     _______     _______
; N d'aiguillage = 0  16 sur 4 bits = 8*RELAIS1 + 4*RELAIS0 + 2*RELAIS4 + 1*RELAIS5
;                                bit n     3           2           1           0            
;
;
; -------RELAIS ON-------RELAIS ON-------Tempd-------RELAIS OFF-------Tempc---------RELAIS OFF
;        R0+R4+R4         R6/R3        temps  de       R6/R3        temps de        R0+R4+R4
;      +R2(Position)      GO 1/2       dcharge C      GO 1/2       charge C      +R2(Position)
;
mem2	btfsc	aigbrut,1
	bcf	RELAIS4		; Position  droit ou dvi = RELAIS2.
	btfss	aigbrut,1
	bsf	RELAIS4

	btfsc	aigbrut,2
	bcf	RELAIS0
	btfss	aigbrut,2
	bsf	RELAIS0

	btfsc	aigbrut,0
	bcf	RELAIS5
	btfss	aigbrut,0
	bsf	RELAIS5

	btfss	aigbrut,3
	bcf	RELAIS1
	btfsc	aigbrut,3
	bsf	RELAIS1

	btfss	aigbrut,bitPOS1	; Position de l'aiguillage, droit ou dvi.
	bcf	RELAIS2		
	btfsc	aigbrut,bitPOS1
	bsf	RELAIS2

	clrf	tmp1		; Tempo avant d'activer le relais GO,
wait2	decfsz	tmp1,f		; le temps que les autres relais soient bien positionns
	goto	wait2

	btfss	aigbrut,bitGO12
	bsf	RELAISGO1	; GO_1 si n aig de  0  15
	btfsc	aigbrut,bitGO12
	bsf	RELAISGO2	; GO_2 si n aig de 16  31
	call	tempd		; Tempo pendant la dcharge du condensateur.

	bcf	PORTB,4		; On teind les points des afficheurs de l'metteur branchs sur le port 4
	bcf	RELAIS0		; Raz des autres relais ici pour faire moins de bruit de commutation.
	bcf	RELAIS1
	bcf	RELAIS2
	bcf	RELAIS4
	bcf	RELAIS5
	bcf	RELAISGO1	; On charge le condensateur quand les autres relais sont dsactivs,
	bcf	RELAISGO2	; on limite ainsi le courant consomm.
	call	tempc		; Tempo pendant la charge du condensateur.

	bsf	INDF,1		; FSR pointe sur l'aiguillage en cours. Positionne : aipos[x] Aiguillage dja utilis !
	bcf	INDF,0
	btfsc	aigbrut,bitPOS1
	bsf	INDF,0		; Positionne : aipos[x] Position (1)!

aigsuit	bcf	PORTB,4		; On teind les points des afficheurs de l'metteur branchs sur le port 4.
	incf	tabind,f	; On passe  l'lment suivant dans la table.
	btfss	numtable,0	; Numtable = 0 ou 1
	goto	bcltbl1
	goto	bcltbl2
	
;*******************************************************************

tblfin	bcf	RELAISGO1	; Toutes les sorties  0 pour ne plus alimenter les relais.
	bcf	RELAISGO2
	bcf	RELAIS0
	bcf	RELAIS1
	bcf	RELAIS2
	bcf	RELAIS4
	bcf	RELAIS5
	goto	carclr		; On fait une raz de tout.


;*******************************************************************
;*******************************************************************
interrupt return 		; Pas d'interruption d'utilise.

;*******************************************************************
; Routines de temporisation pour la commande des aiguillages
; On boucle dans cette routine un certain temps.
;
; Routine charge   = Dure de recharge entre deux impulsions sur 2 bits suivant tcharge 0=100 ms, 1=200ms, 2=300ms, 3=400ms.
; Routine decharge = Dure de dcharge de l'impulsion sur 2 bits suivant tdecharge      0=100 ms, 1=200ms, 2=300ms, 3=400ms.
;
; Table de dlais = tmpd10  l'adresse 0x700
;   Entre w = 0,1,2,3 pour le temps de charge
;   Entre w = 4,5,6,7 pour le temps de dcharge

tempc	movlw	0x07
	movwf	PCLATH
	movf	tcharge,w
	call	tmpd10			; Tableau en 0x700
	movwf	cpt1
tmpd1	movlw	13			; Pour avoir une boucle globale de 10 msec par unit de tcharge/tdecharge.
	movwf	cpt2
tmpd2	clrf	cpt3
tmpd3	decfsz	cpt3			; Dcrmenter compteur cpt3 sur 256 boucles
	goto	tmpd3			; Si pas 0, boucler
	decfsz	cpt2,f			; Dcrmenter compteur cpt2 sur 14 boucles
	goto	tmpd2			; Si pas 0, boucler
	decfsz	cpt1,f			; Dcrmenter compteur cpt1 sur N * 10 msec.
	goto	tmpd1			; Si pas 0, boucler
	return

tempd	movlw	0x07
	movwf	PCLATH
	movf	tdecharge,w
	addlw	4			; w = w + 4 
	call	tmpd10			; Tableau en 0x700.
	movwf	cpt1
	goto	tmpd1

;*********************************************************************************
; tabaig1 et tabaig2 = Deux tables de description des aiguillages par n de garage
;
; 1er lment = Liste des aiguillages  activer pour le garage n 0
; Le garage n0 n'existe pas, mais permet de placer l'aiguillage d'accs au faisceau sur la voie principale.
;
; Contenu de la table :
;   naiguillage + position de l'aiguillage(POS0/POS1)
;   On alimente en dernier la voie de garage avec la ligne "retlw 0+POS1".
;   POSFIN    = Repre de fin d'action pour le garage en cours.
;   POSFINFIN = Repre de fin de table
;
; Si le premier tableau est satur (plus de 255 lments), il faut remplir le second tableau.
; Ne pas dcrire de garage  cheval sur les deux tables.

	ORG	0x500		; Adresse du tableau 
tabaig1	addwf	PCL,f		; Ajouter w  PCL

	retlw	0+POS0		; Garage 0 = Aucun garage aliment => Relais gnral d'alimentation = aiguillage n0
	retlw	31+POS1		; Aiguillage voie principale
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 1
	retlw	1+POS0
	retlw	2+POS0
	retlw	3+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 2		
	retlw	1+POS0
	retlw	2+POS0
	retlw	3+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 3		
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 4		
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 5		
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS1
	retlw	6+POS0
	retlw	7+POS0
	retlw	8+POS0
	retlw	9+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 6		
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS1
	retlw	6+POS0
	retlw	7+POS0
	retlw	8+POS0
	retlw	9+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garages
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 7		
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS1
	retlw	6+POS0
	retlw	7+POS0
	retlw	8+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 8	
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS1
	retlw	6+POS0
	retlw	7+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 9	
	retlw	1+POS0
	retlw	2+POS1
	retlw	4+POS1
	retlw	5+POS1
	retlw	6+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 10
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS0
	retlw	15+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 11
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS0
	retlw	15+POS1
	retlw	16+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 12	
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS0
	retlw	15+POS1
	retlw	16+POS1
	retlw	17+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 13
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS0
	retlw	15+POS1
	retlw	16+POS1
	retlw	17+POS1
	retlw	18+POS0
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 14
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS0
	retlw	15+POS1
	retlw	16+POS1
	retlw	17+POS1
	retlw	18+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 15
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS0
	retlw	14+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 16		
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS0
	retlw	13+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 17		
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS0
	retlw	12+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	0+POS0		; Garage 18		
	retlw	1+POS1
	retlw	10+POS0
	retlw	11+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin de garage 

	retlw	POSFINFIN	; Fin de table n1 

; Les lignes suivantes permettent de sortir une erreur  la compilation si la table dborde
        IF ((HIGH ($)) != (HIGH (tabaig1+1)))
            ERROR "La table tabaig1 dpasse 256 octets!"
        ENDIF

;*******************************************************************
	ORG	0x600		; Adresse du second tableau
tabaig2	addwf	PCL,f		; Ajouter w  PCL

	retlw	0+POS0		
	retlw	1+POS1		; Garage 19
	retlw	10+POS1
	retlw	31+POS0		; Aiguillage voie principale
	retlw	0+POS1		; Alimentation du garage
	retlw	POSFIN		; Fin du garage 
	retlw	POSFINFIN	; Fin de table n 2

; Les lignes suivantes permettent de sortir une erreur  la compilation si la table dborde
        IF ((HIGH ($)) != (HIGH (tabaig2+1)))
            ERROR "La table tabaig2 dpasse 256 octets"
        ENDIF

;*******************************************************************
; Table de calcul de dlais 
; Entre w = 0,1,2,3 pour le temps de charge du condensateur
; Entre w = 4,5,6,7 pour le temps de dcharge du condensateur
; Retour w = Temps de pause en dizaine de ms  (Exemple si retour=10 => 100 msec)

	ORG	0x700		; Adresse du tableau des temporisation.
tmpd10	addwf	PCL,f		; ajouter w  PCL

	retlw	10		; Retour pour le temps de charge pour w = 0,1,2,3, en dizaine de ms
	retlw	20
	retlw	30
	retlw	40

	retlw	10		; Retour pour le temps de dcharge pour w = 4,5,6,7, en dizaine de ms
	retlw	20
	retlw	30
	retlw	40
	END